/*
 * Copyright 2019, Karl Nelson
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */
package org.jpype.pickle;

import java.io.IOException;
import java.io.ObjectInputStream;

public class Decoder
{
  ByteBufferInputStream bb;
  ObjectInputStream ois = null;

  public Decoder() throws IOException
  {
    bb = new ByteBufferInputStream();
  }

  public Object unpack(byte[] data) throws IOException, ClassNotFoundException
  {
    bb.put(data);
    if (ois == null)
      ois = new ObjectInputStream(bb);
    return ois.readObject();
  }
}
